import unittest

class TestUtilsMethods(unittest.TestCase):

    def test_add_secure_hash(self):
        from nps_sdk.utils import add_secure_hash
        params = {'psp_QueryCriteriaId': '100409',
                  'psp_QueryCriteria': 'T',
                  'psp_Version': '2.2',
                  'psp_MerchantId': 'psp_test',
                  'psp_PosDateTime': '2016-12-01 12:00:00'}

        secret_key = "IeShlZMDk8mp8VA6vy41mLnVggnj1yqHcJyNqIYaRINZnXdiTfhF0Ule9WNAUCR6"

        resp = add_secure_hash(params, secret_key)
        self.assertEqual('d2f455e71232d7842c739196f6fa25f7', resp.get('psp_SecureHash'))
        self.assertEqual('psp_test', resp.get('psp_MerchantId'))
        self.assertEqual('100409', resp.get('psp_QueryCriteriaId'))


    def test_create_secure_hash(self):
        from nps_sdk.utils import _create_secure_hash
        secret_key = "IeShlZMDk8mp8VA6vy41mLnVggnj1yqHcJyNqIYaRINZnXdiTfhF0Ule9WNAUCR6"

        params = {'psp_QueryCriteriaId': '100409',
                  'psp_QueryCriteria': 'T',
                  'psp_Version': '2.2',
                  'psp_MerchantId': 'psp_test',
                  'psp_PosDateTime': '2016-12-01 12:00:00'}

        resp = _create_secure_hash(params, secret_key)
        self.assertEqual('d2f455e71232d7842c739196f6fa25f7', resp)

    def test_add_extra_info(self):
        from nps_sdk.utils import add_extra_info
        params = {'psp_QueryCriteriaId': '100409',
                  'psp_QueryCriteria': 'T',
                  'psp_Version': '2.2',
                  'psp_MerchantId': 'psp_test',
                  'psp_PosDateTime': '2016-12-01 12:00:00'}

        resp = add_extra_info('PayOnline_2p',params=params)
        self.assertTrue('psp_MerchantAdditionalDetails' in resp.keys())
        self.assertTrue('SdkInfo' in resp.get('psp_MerchantAdditionalDetails').keys())
        self.assertEqual('Python 1.0', resp.get('psp_MerchantAdditionalDetails').get('SdkInfo'))

    def _test_validate_size(self):
        from nps_sdk.utils import _validate_size
        from nps_sdk.conf.sanititize_struc import key_config
        value = 'Python 1.0'
        key = 'psp_MerchantAdditionalDetails.SdkInfo'
        resp = _validate_size(value=value, k=key)
        self.assertGreaterEqual(key_config.get(key+".max_length"), len(resp))

    def test_mask_credit_card(self):
        from nps_sdk.utils import _mask_c_number
        card_number = """<psp_CardNumber xsi:type="ns2:string">4507990000000011</psp_CardNumber>"""
        resp = _mask_c_number(card_number)
        self.assertEqual("""<psp_CardNumber xsi:type="ns2:string">450799******0011</psp_CardNumber>""", resp)

    def test_mask_cvc(self):
        from nps_sdk.utils import _mask_cvc
        cvc = """<psp_CardSecurityCode xsi:type="ns2:string">123</psp_CardSecurityCode>"""
        resp = _mask_cvc(cvc)
        self.assertEqual("""<psp_CardSecurityCode xsi:type="ns2:string">***</psp_CardSecurityCode>""", resp)

    def test_mask_exp_date(self):
        from nps_sdk.utils import _mask_exp_date
        card_exp_date = """<psp_CardExpDate xsi:type="ns2:string">1712</psp_CardExpDate>"""
        resp = _mask_exp_date(card_exp_date)
        self.assertEqual("""<psp_CardExpDate xsi:type="ns2:string">****</psp_CardExpDate>""", resp)


    def test_mask_data(self):
        from nps_sdk.utils import _mask_data
        datos = """
        <psp_CardExpDate xsi:type="ns2:string">1712</psp_CardExpDate>
        <psp_CardSecurityCode xsi:type="ns2:string">123</psp_CardSecurityCode>
        <psp_CardNumber xsi:type="ns2:string">4507990000000011</psp_CardNumber>
        """
        espected = """
        <psp_CardExpDate xsi:type="ns2:string">****</psp_CardExpDate>
        <psp_CardSecurityCode xsi:type="ns2:string">***</psp_CardSecurityCode>
        <psp_CardNumber xsi:type="ns2:string">450799******0011</psp_CardNumber>
        """
        resp = _mask_data(datos)
        self.assertEqual(espected, resp)

    def test_find_c_numbers(self):
        from nps_sdk.utils import _find_c_numbers
        datos = """
                <psp_CardExpDate xsi:type="ns2:string">1712</psp_CardExpDate>
                <psp_CardSecurityCode xsi:type="ns2:string">123</psp_CardSecurityCode>
                <psp_CardNumber xsi:type="ns2:string">4507990000000011</psp_CardNumber>
                """

        resp = _find_c_numbers(datos)
        self.assertEqual(['4507990000000011</psp_CardNumber>'], resp)

    def test_find_exp_date(self):
        from nps_sdk.utils import _find_exp_date
        datos = """
                <psp_CardExpDate xsi:type="ns2:string">1712</psp_CardExpDate>
                <psp_CardSecurityCode xsi:type="ns2:string">123</psp_CardSecurityCode>
                <psp_CardNumber xsi:type="ns2:string">4507990000000011</psp_CardNumber>
                """
        resp = _find_exp_date(datos)
        self.assertEqual(['1712</psp_CardExpDate>'], resp)

    def test_find_cvc(self):
        from nps_sdk.utils import _find_cvc
        datos = """
                <psp_CardExpDate xsi:type="ns2:string">1712</psp_CardExpDate>
                <psp_CardSecurityCode xsi:type="ns2:string">123</psp_CardSecurityCode>
                <psp_CardNumber xsi:type="ns2:string">4507990000000011</psp_CardNumber>
                """
        resp = _find_cvc(datos)
        self.assertEqual(['123</psp_CardSecurityCode>'], resp)


class TestServices(unittest.TestCase):

    def test_get_not_add_services(self):
        from nps_sdk.services import get_merch_det_not_add_services
        espected = ['QueryTxs', 'Refund', 'SimpleQueryTx', 'Capture', 'ChangeSecretKey', 'NotifyFraudScreeningReview', 'GetIINDetails', 'QueryCardNumber', 'CreatePaymentMethod', 'CreatePaymentMethodFromPayment', 'RetrievePaymentMethod', 'UpdatePaymentMethod', 'DeletePaymentMethod', 'CreateCustomer', 'RetrieveCustomer', 'UpdateCustomer', 'DeleteCustomer', 'RecachePaymentMethodToken', 'CreatePaymentMethodToken', 'RetrievePaymentMethodToken', 'CreateClientSession', 'GetInstallmentsOptions', 'QueryCardDetails']
        resp = get_merch_det_not_add_services()
        self.assertEqual(espected, resp)

class TestIssues(unittest.TestCase):

    def test_secure_hash_w_list_in_root_node(self):
        from nps_sdk.utils import _create_secure_hash

        key = "IeShlZMDk8mp8VA6vy41mLnVggnj1yqHcJyNqIYaRINZnXdiTfhF0Ule9WNAUCR6"

        params = {
            "psp_Version": '2.2',
            "psp_MerchantId": 'psp_test',
            "psp_TxSource": 'WEB',
            "psp_MerchOrderId": 'ORDER999qdw',
            "psp_ReturnURL": 'http://localhost/',
            "psp_FrmLanguage": 'es_AR',
            "psp_Amount": 15050,
            "psp_Currency": '032',
            "psp_Country": 'ARG',
            "psp_Product": 14,
            "psp_PosDateTime": '2016-12-01 12:00:00',
            "psp_Transactions": [{
                "psp_MerchantId": 'psp_test',
                "psp_MerchTxRef": 'ORDER36qx6-3',
                "psp_Product": 14,
                "psp_Amount": 10000,
                "psp_NumPayments": 1
            },{
                "psp_MerchantId": 'psp_test',
                "psp_MerchTxRef": 'ORDER1qd66-3',
                "psp_Product": 14,
                "psp_Amount": 5050,
                "psp_NumPayments": 1
            }]
        }

        resp = _create_secure_hash(params=params, secret_key=key)
        self.assertEqual("a4defbaaaf41d581c8a35014820404df", resp)


    def test_issue_3930_error_500_on_p2p_with_updated_costumer_on_dev(self):
        from nps_sdk.constants import DEVELOPMENT_ENV
        import nps_sdk
        import logging
        import uuid
        merch_ref = uuid.uuid4()

        nps_sdk.Configuration.configure(environment=DEVELOPMENT_ENV,
                                        secret_key="IeShlZMDk8mp8VA6vy41mLnVggnj1yqHcJyNqIYaRINZnXdiTfhF0Ule9WNAUCR6",
                                        log_level=logging.INFO, debug=True, cert_verify_peer=False)

        sdk = nps_sdk.Nps()

        merchant_id = 'psp_test'

        params = {'psp_AccountCreatedAt': '2010-10-23',
                  'psp_EmailAddress': 'fedevalle@inclufin.com',
                  'psp_MerchantId': merchant_id,
                  'psp_Person': {'FirstName': 'Federico',
                                 'LastName': 'del Valle',
                                 'PhoneNumber1': '+5491164838755'},
                  'psp_PosDateTime': '2017-02-09 18:42:43',
                  'psp_Version': '2.2'}

        create_customer_response = sdk.create_customer(params)


        params = {
            'psp_AccountCreatedAt': '2010-10-23',
            'psp_Address': {
                'AdditionalInfo': '2 A',
                'City': 'Miami',
                'Country': 'USA',
                'HouseNumber': '1245',
                'StateProvince': 'Florida',
                'Street': 'Av. Collins',
                'ZipCode': '33140'
            },
            'psp_AlternativeEmailAddress': 'jdoe@example.com',
            'psp_CustomerId': create_customer_response.psp_CustomerId,
            'psp_EmailAddress': 'jhon.doe@example.com',
            'psp_MerchantId': merchant_id,
            'psp_PaymentMethod': {
                'CardInputDetails': {
                    'ExpirationDate': '1909',
                    'HolderName': 'VISA',
                    'Number': '4242424242424242',
                    'SecurityCode': '9822'
                },
                'Product': '14'
            },
            'psp_Person': {
                'DateOfBirth': '1979-01-12',
                'FirstName': 'John',
                'Gender': 'M',
                'IDNumber': '54111111',
                'IDType': '200',
                'LastName': 'Doe',
                'MiddleName': 'Michael',
                'Nationality': 'ARG',
                'PhoneNumber1': '+1 011 11111111',
                'PhoneNumber2': '+1 011 22222222'
            },
            'psp_PosDateTime': '2008-01-12 13:05:00',
            'psp_Version': '2.2'
        }

        update_customer_response = sdk.update_customer(params)

        params = {
            'psp_MerchOrderId': merch_ref,
            'psp_VaultReference': {
                'CustomerId': update_customer_response.psp_CustomerId
            },
            'psp_MerchTxRef': merch_ref,
            'psp_PosDateTime': '2017-03-31 16:14:06',
            'psp_NumPayments': '1',
            'psp_Currency': '032',
            'psp_TxSource': 'WEB',
            'psp_Product': '14',
            'psp_Country': 'ARG',
            'psp_Version': '2.2',
            'psp_MerchantId': merchant_id,
            'psp_Amount': "1000"
        }

        p2p_response = sdk.pay_online_2p(params)


        self.assertEqual("2", p2p_response.psp_ResponseCod)



# -*- coding: utf-8 -*-
import sys
try:
    reload(sys)
    sys.setdefaultencoding('utf8')
except NameError:
    pass


class TransactionException(Exception):
    msg = """
            Response code: %s
            Response Message: %s
            Extended Response Message: %s
            """

    def __init__(self, response):
        self._resp_code = response.psp_ResponseCod.encode('utf-8')
        self._resp_msg = response.psp_ResponseMsg.encode('utf-8')
        self._resp_msg_ext = response.psp_ResponseExtended.encode('utf-8')
        self._response = response
        Exception.__init__(self,
                           TransactionException.msg % (self._resp_code,
                                                       self._resp_msg,
                                                       self._resp_msg_ext))

    def get_message(self):
        return self._response.psp_ResponseMsg

    def get_extended_message(self):
        return self._response.psp_ResponseExtended

    def get_raw_response(self):
        return self._response


class RejectedException(TransactionException):
    pass


class DeclinedException(TransactionException):
    pass


class MethodNotFoundException(Exception):
    msg = """El metodo %s no fué encontrado"""

    def __init__(self, name):
        Exception.__init__(self, MethodNotFoundException.msg % (name))


class ApiException(Exception):
    msg = """
            El tiempo de ejecución há expirado
            """

    def __init__(self):
        Exception.__init__(self, ApiException.msg)


class LogException(Exception):
    msg = """
            DEBUG level is now allowed in PRODUCTION ENVIRONMENT
            """

    def __init__(self):
        Exception.__init__(self, LogException.msg)


class EnvironmentNotFound(Exception):
    msg = """
            El ambiente seleccionado es invalido.
            los ambientes validos son
            0: PRODUCCION
            1: STAGING
            2: SANDBOX
            """

    def __init__(self):
        Exception.__init__(self, EnvironmentNotFound.msg)


from nps_sdk import constants
from nps_sdk import errors
import logging


class Configuration(object):

    @staticmethod
    def configure(environment=constants.SANDBOX_ENV,
                  secret_key="",
                  debug=False,
                  timeout=60,
                  cert_verify_peer=True,
                  log_level=logging.INFO,
                  **kwargs):
        Configuration.environment = environment
        Configuration.secret_key = secret_key
        Configuration.debug = debug
        Configuration.timeout = timeout
        Configuration.cert_verify_peer = True
        Configuration.log_level = log_level
        Configuration.certificate = kwargs.get('cert', cert_verify_peer)
        Configuration.c_key = kwargs.get('key_cert')
        Configuration.sanitize = kwargs.get('sanitize', True)
        Configuration.log_file = kwargs.get('log_file')
        Configuration.proxy_url = kwargs.get('proxy_url')
        Configuration.proxy_username = kwargs.get('proxy_username')
        Configuration.proxy_password = kwargs.get('proxy_password')

    @staticmethod
    def get_wsdl():
        try:
            if Configuration.environment < 0:
                raise IndexError
            envs = (constants._PRODUCTION_URL,
                    constants._STAGING_URL,
                    constants._SANDBOX_URL,
                    constants._DEVELOPMENT_URL)
            return envs[Configuration.environment]
        except IndexError:
            raise errors.EnvironmentNotFound




PAY_ONLINE_2P = "PayOnLine_2p"
AUTHORIZE_2P = "Authorize_2p"
QUERY_TXS = "QueryTxs"
SIMPLE_QUERY_TX = "SimpleQueryTx"
REFUND = "Refund"
CAPTURE = "Capture"
AUTHORIZE_3P = "Authorize_3p"
BANK_PAYMENT_3P = "BankPayment_3p"
CASH_PAYMENT_3P = "CashPayment_3p"
CHANGE_SECRET_KEY = "ChangeSecretKey"
FRAUD_SCREENING = "FraudScreening"
NOTIFY_FRAUD_SCREENING_REVIEW = "NotifyFraudScreeningReview"
PAY_ONLINE_3P = "PayOnLine_3p"
SPLIT_AUTHORIZE_3P = "SplitAuthorize_3p"
SPLIT_PAY_ONLINE_3P = "SplitPayOnLine_3p"
BANK_PAYMENT_2P = "BankPayment_2p"
SPLIT_PAY_ONLINE_2P = "SplitPayOnLine_2p"
SPLIT_AUTHORIZE_2P = "SplitAuthorize_2p"
QUERY_CARD_NUMBER = "QueryCardNumber"
QUERY_CARD_DETAILS = "QueryCardDetails"
GET_IIN_DETAILS = "GetIINDetails"

CREATE_PAYMENT_METHOD = "CreatePaymentMethod"
CREATE_PAYMENT_METHOD_FROM_PAYMENT = "CreatePaymentMethodFromPayment"
RETRIEVE_PAYMENT_METHOD = "RetrievePaymentMethod"
UPDATE_PAYMENT_METHOD = "UpdatePaymentMethod"
DELETE_PAYMENT_METHOD = "DeletePaymentMethod"
CREATE_CUSTOMER = "CreateCustomer"
RETRIEVE_CUSTOMER = "RetrieveCustomer"
UPDATE_CUSTOMER = "UpdateCustomer"
DELETE_CUSTOMER = "DeleteCustomer"
RECACHE_PAYMENT_METHOD_TOKEN = "RecachePaymentMethodToken"
CREATE_PAYMENT_METHOD_TOKEN = "CreatePaymentMethodToken"
RETRIEVE_PAYMENT_METHOD_TOKEN = "RetrievePaymentMethodToken"
CREATE_CLIENT_SESSION = "CreateClientSession"
GET_INSTALLMENTS_OPTIONS = "GetInstallmentsOptions"


def get_merch_det_not_add_services():
    return [QUERY_TXS,
            REFUND,
            SIMPLE_QUERY_TX,
            CAPTURE,
            CHANGE_SECRET_KEY,
            NOTIFY_FRAUD_SCREENING_REVIEW,
            GET_IIN_DETAILS,
            QUERY_CARD_NUMBER,
            CREATE_PAYMENT_METHOD,
            CREATE_PAYMENT_METHOD_FROM_PAYMENT,
            RETRIEVE_PAYMENT_METHOD,
            UPDATE_PAYMENT_METHOD,
            DELETE_PAYMENT_METHOD,
            CREATE_CUSTOMER,
            RETRIEVE_CUSTOMER,
            UPDATE_CUSTOMER,
            DELETE_CUSTOMER,
            RECACHE_PAYMENT_METHOD_TOKEN,
            CREATE_PAYMENT_METHOD_TOKEN,
            RETRIEVE_PAYMENT_METHOD_TOKEN,
            CREATE_CLIENT_SESSION,
            GET_INSTALLMENTS_OPTIONS,
            QUERY_CARD_DETAILS]



from nps_sdk.configuration import Configuration
from nps_sdk import utils, constants
from requests.exceptions import ReadTimeout, ConnectTimeout
from nps_sdk.utils import RequestsTransport, LogPlugin, MaskedLogPlugin
from nps_sdk.errors import ApiException, LogException
from suds import client
import logging
from nps_sdk.file_adapter import FileAdapter
import requests
from suds.cache import NoCache
from requests.auth import HTTPProxyAuth
import requests

class SoapClient(object):
    def __init__(self):
        self._setup()

    def _setup(self):
        plugings = []
        if Configuration.debug:
            if Configuration.log_file is not None:
                logging.basicConfig(filename=Configuration.log_file, level=Configuration.log_level,
                                    format=utils.get_log_format())
            else:
                logging.basicConfig(level=Configuration.log_level,
                                    format=utils.get_log_format())

            if Configuration.log_level == logging.DEBUG and Configuration.environment ==  constants.PRODUCTION_ENV:
                raise LogException

            if Configuration.log_level > logging.DEBUG or Configuration.log_level == 0:
                plugings.append(MaskedLogPlugin())
            else:
                plugings.append(LogPlugin())


        s = requests.Session()
        s.mount('file://', FileAdapter())
        if Configuration.proxy_url:
            s.proxies = { 'https': Configuration.proxy_url }
            if Configuration.proxy_username:
                s.auth= HTTPProxyAuth(Configuration.proxy_username, Configuration.proxy_password)

        if Configuration.certificate and Configuration.c_key:
            s.cert=(Configuration.certificate, Configuration.c_key)
        else:
            if not Configuration.certificate:
                pass
                #from requests.packages.urllib3.exceptions import InsecureRequestWarning
                #requests.packages.urllib3.disable_warnings(InsecureRequestWarning)

            s.verify = Configuration.certificate

        t = RequestsTransport(s, timeout=Configuration.timeout)
        self._client = client.Client(Configuration.get_wsdl().strip(), plugins=plugings, transport=t, cache=NoCache())

    def _soap_call(self, service, params):
        try:
            params = utils.add_extra_info(service, params)
            if Configuration.sanitize:
                params = utils._check_sanitize(params=params, is_root=True)
            if not params.get('psp_ClientSession'):
                params = utils.add_secure_hash(params, Configuration.secret_key)

            response = getattr(self._client.service, service)(params)
            return response
        except ReadTimeout:
            raise ApiException
        except ConnectTimeout:
            raise ApiException



version = '1.1.6'

from setuptools import setup, find_packages

setup(
  name = 'nps_sdk',
  packages = find_packages(),
  version = version,
  description = 'A Python SDK for Ingenico ePayments - NPS LatAm Services',
  author = 'Gustavo Diaz',
  author_email = 'gustavo.diaz@ingenico.com',
  url = 'https://github.com/Ingenico-NPS-Latam/nps-sdk-python',
  keywords = ['ingenico', 'payments', 'npssdk', 'nps-sdk'],
  classifiers = [
        "Development Status :: 4 - Beta",
        "Intended Audience :: Developers",
        "Operating System :: OS Independent",
        "Programming Language :: Python",
        "Programming Language :: Python :: 2.7",
        "Programming Language :: Python :: 3",
        "Programming Language :: Python :: 3.3",
        "Programming Language :: Python :: 3.4",
        "Programming Language :: Python :: 3.5",
        "Programming Language :: Python :: Implementation :: PyPy",
        "Topic :: Software Development :: Libraries :: Python Modules"],
  install_requires=['suds-jurko==0.6', 'requests==2.9.1', 'six==1.10.0'],
  package_data={ 'nps_sdk': ['wsdl/staging.wsdl',
                         'wsdl/sandbox.wsdl',
                         'wsdl/production.wsdl',
                         'wsdl/development.wsdl'] }
)